/*
 * Copyright (C) 2024 usbharu
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package dev.usbharu.hideout.application.infrastructure.exposed

class PaginationList<T, ID>(list: List<T>, val next: ID?, val prev: ID?) : List<T> by list

fun <T, ID> PaginationList<T, ID>.toHttpHeader(
    nextBlock: (string: String) -> String,
    prevBlock: (string: String) -> String
): String? {
    val mutableListOf = mutableListOf<String>()
    if (next != null) {
        mutableListOf.add("<${nextBlock(this.next.toString())}>; rel=\"next\"")
    }
    if (prev != null) {
        mutableListOf.add("<${prevBlock(this.prev.toString())}>; rel=\"prev\"")
    }

    if (mutableListOf.isEmpty()) {
        return null
    }

    return mutableListOf.joinToString(", ")
}
