/*
 * Copyright (C) 2024 usbharu
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package dev.usbharu.hideout.core.infrastructure.springframework.httpsignature

import org.springframework.security.core.GrantedAuthority
import org.springframework.security.core.userdetails.User
import java.io.Serial

class HttpSignatureUser(
    username: String,
    val domain: String,
    val id: Long,
    credentialsNonExpired: Boolean,
    accountNonLocked: Boolean,
    authorities: MutableCollection<out GrantedAuthority>?
) : User(
    username,
    "",
    true,
    true,
    credentialsNonExpired,
    accountNonLocked,
    authorities
) {

    override fun equals(other: Any?): Boolean {
        if (this === other) return true
        if (other !is HttpSignatureUser) return false
        if (!super.equals(other)) return false

        if (domain != other.domain) return false
        if (id != other.id) return false

        return true
    }

    override fun hashCode(): Int {
        var result = super.hashCode()
        result = 31 * result + domain.hashCode()
        result = 31 * result + id.hashCode()
        return result
    }

    override fun toString(): String {
        return "HttpSignatureUser(" +
            "domain='$domain', " +
            "id=$id" +
            ")" +
            " ${super.toString()}"
    }

    companion object {
        @Serial
        private const val serialVersionUID: Long = -3330552099960982997L
    }
}
